import fs from 'node:fs';
import path from 'node:path';
import { checkFileExists, AssistantConfig, AssistantConfigData, parseHomeArg, parseHelpArg } from '@/module/assistant/index.ts';
import { chalk } from '@/module/chalk.ts';
import { HttpsPem } from '@/module/assistant/https/sign.ts';
export { parseHomeArg, parseHelpArg };
export type AssistantInitOptions = {
  path?: string;
  init?: boolean;
};
/**
 * 助手初始化类
 * @class AssistantInit
 */
export class AssistantInit extends AssistantConfig {
  constructor(opts?: AssistantInitOptions) {
    const configDir = opts?.path || process.cwd();
    super({
      configDir,
      init: false,
    });
    if (opts?.init) {
      this.init();
    }
  }

  async init() {
    // 1. 检查助手路径是否存在
    if (!this.checkConfigPath()) {
      console.log(chalk.blue('助手路径不存在，正在创建...'));
      super.init();
    } else {
      super.init();
      const assistantConfig = this;
      console.log(chalk.yellow('助手路径已存在'), chalk.green(assistantConfig.configDir));
    }
    this.createAssistantConfig();
    this.createEnvConfig();
    this.createOtherConfig();
  }
  checkConfigPath() {
    const assistantPath = path.join(this.configDir, 'assistant-app', 'assistant-config.json');
    return checkFileExists(assistantPath);
  }
  createEnvConfig() {
    const env = this.configPath?.envConfigPath;
    // 创建助手环境配置文件 env
    if (!checkFileExists(env, true)) {
      fs.writeFileSync(env, '# 环境配置文件\n');
      console.log(chalk.green('助手环境配置.env文件创建成功'));
    }
  }
  createOtherConfig() {
    const appsConfig = this.configPath?.appsConfigPath;
    // 创建助手应用配置文件 apps
    if (!checkFileExists(appsConfig, true)) {
      fs.writeFileSync(appsConfig, JSON.stringify({ description: 'apps manager.', list: [] }));
      console.log(chalk.green('助手应用配置文件 apps.json 创建成功'));
    }
    // create pem dir //
    const pemDir = path.join(this.configPath?.configDir, 'pem');
    if (!checkFileExists(pemDir)) {
      new HttpsPem(this);
      console.log(chalk.green('助手证书目录创建成功'));
    }
  }
  createAssistantConfig() {
    const assistantPath = this.configPath?.configPath;
    // 创建助手配置文件 assistant-config.json
    if (!checkFileExists(assistantPath, true)) {
      this.setConfig(this.getDefaultInitAssistantConfig());
      console.log(chalk.green('助手配置文件assistant-config.json创建成功'));
    }
  }
  initPnpm() {
    const pnpmPath = path.join(this.configDir, 'assistant-app', 'pnpm.yaml');
    let create = false;
    if (!checkFileExists(pnpmPath, true)) {
      create = true;
      fs.writeFileSync(
        pnpmPath,
        `packages:
  - 'apps/**/*'
  - 'pages/**/*'
 `,
      );
      console.log(chalk.green('助手 pnpm.yaml 文件创建成功'));
    }
    const packagePath = path.join(this.configDir, 'assistant-app', 'package.json');
    if (!checkFileExists(packagePath, true)) {
      create = true;
      fs.writeFileSync(
        packagePath,
        `{
  "name": "assistant-app",
  "version": "1.0.0",
  "description": "assistant-app package pnpm, node pkgs projects",
  "type": "module",
  "scripts": {
    "start": "pm2 start apps/code-center/dist/app.mjs --name code-center",
    "proxy": "pm2 start apps/page-proxy/dist/app.mjs --name page-proxy"
  },
  "keywords": [],
  "author": "",
  "license": "ISC",
  "dependencies": {
    "@kevisual/router": "^0.0.20",
    "@kevisual/use-config": "^1.0.17",
    "ioredis": "^5.6.1",
    "minio": "^8.0.5",
    "pg": "^8.16.0",
    "pm2": "^6.0.6",
    "sequelize": "^6.37.7",
    "sqlite3": "^5.1.7",
    "socket.io": "^4.8.1",
    "dotenv": "^16.5.0"
  }
}
`,
      );
      console.log(chalk.green('助手 package.json 文件创建成功'));
    }
    return {
      create,
    };
  }
  protected getDefaultInitAssistantConfig() {
    return {
      description: '助手配置文件',
      home: '/root/center',
      proxy: [],
      apiProxyList: [],
      share: {
        enabled: false,
        name: 'abc',
        url: 'https://kevisual.cn/ws/proxy',
      },
      watch: {
        enabled: true,
      },
    } as AssistantConfigData;
  }
}
